#!/bin/bash
######################################################################
#
#   tcar - The CentOS Artwork Repository
#   Copyright © 2014 The CentOS Artwork SIG
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation; either version 2 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#   Alain Reguera Delgado <alain.reguera@gmail.com>
#   39 Street No. 4426 Cienfuegos, Cuba.
#
######################################################################

#======================================================================
# Script Internationalization
#======================================================================

# Set function environments required by GNU gettext system.
. gettext.sh

# Set the script text domain. This information is used by gettext
# system to retrieve translated strings from machine object (MO) files
# with this name. This variable is reset based on the common function
# or module in turn.
declare -x TEXTDOMAIN="${SCRIPT_NAME}"

# Set the script text domain directory. This information is used by
# gettext system to know where the machine object (MO) files are
# stored in. The value of variable is the same for both common
# functions and individual modules.
declare -x TEXTDOMAINDIR=/usr/share/locale

# Set the script language information using the LC format. This format
# shows both language and country information (e.g., `es_ES').
declare -xr SCRIPT_LANG_LC=$(echo ${LANG} | cut -d'.' -f1)

# Set the script language information using the LL format. This format
# shows only the language information (e.g., `es').
declare -xr SCRIPT_LANG_LL=$(echo ${SCRIPT_LANG_LC} | cut -d'_' -f1)

# Set the script language information using the CC format. This format
# shows only the country information (e.g., `ES').
declare -xr SCRIPT_LANG_CC=$(echo ${SCRIPT_LANG_LC} | cut -d'_' -f2)

#======================================================================
# Script Paths
#======================================================================

# Base directory where man pages are installed in.
declare -x SCRIPT_MANUALS_BASEDIR=/usr/share/man

# Base directory where configuration files are installed in.
declare -x SCRIPT_CONFIGS_BASEDIR=/etc/${SCRIPT_NAME}

# Default text editor.
declare -x SCRIPT_EDITOR=/usr/bin/vim

# Base directory where automation script modules are installed in.
declare -x SCRIPT_MODULES_BASEDIR=${SCRIPT_BASEDIR}/Modules

# Directory to store temporal files.
declare -x SCRIPT_TEMPDIR=$(mktemp -p /tmp -d ${SCRIPT_NAME}-XXXXXX)

#======================================================================
# Script Flags
#======================================================================

# Set filter flag (-f|--filter).  This flag is mainly used to reduce
# the number of files to process and is interpreted as egrep-posix
# regular expression.  By default, when this flag is not provided, all
# paths in the working copy will match, except files inside hidden
# directories like `.svn' and `.git' that will be omitted.
declare -x SCRIPT_FLAG_FILTER='[[:alnum:]_/-]+'

# Set verbosity flag (-q|--quiet). This flag controls whether tcarlib.sh
# script prints messages or not. By default, all messages are
# suppressed except those directed to standard error.
declare -x SCRIPT_FLAG_QUIET='false'

# Set affirmative flag (-y|--yes). This flag controls whether tcarlib.sh
# script does or does not pass confirmation request points. By
# default, it doesn't.
declare -x SCRIPT_FLAG_YES='false'

# Set debugger flag (-d|--debug). This flag controls whether tcarlib.sh
# script does or does not print debugging information.  The tcarlib.sh
# script prints debug information to standard output.
declare -x SCRIPT_FLAG_DEBUG='false'

#======================================================================
# Script Configuration Files (redefine global variables)
#======================================================================

declare -x SCRIPT_CONFIG_FILE=""
declare -x SCRIPT_CONFIG_FILES="\
    /etc/${SCRIPT_NAME}.conf \
    ${SCRIPT_CONFIGS_BASEDIR}/${SCRIPT_NAME}.conf \
    ${HOME}/.${SCRIPT_NAME}.conf"

for SCRIPT_CONFIG_FILE in ${SCRIPT_CONFIG_FILES};do
    [[ -f ${SCRIPT_CONFIG_FILE} ]] && . ${SCRIPT_CONFIG_FILE}
done

#======================================================================
# Script Global Functions
#======================================================================

declare -x  SCRIPT_LIBRARIES_BASEDIR=$(dirname ${SCRIPT_LIBRARIES})
declare -x  SCRIPT_FUNCTIONS_BASEDIR="${SCRIPT_BASEDIR}"
declare -x  SCRIPT_FUNCTION=''
declare -xa SCRIPT_FUNCTIONS

SCRIPT_FUNCTIONS[0]=$(find ${SCRIPT_LIBRARIES_BASEDIR} -type f -name 'tcarlib_*.sh')
SCRIPT_FUNCTIONS[1]=$(find ${SCRIPT_FUNCTIONS_BASEDIR} -maxdepth 1 \
    -mindepth 1 -type f -name "${SCRIPT_NAME}_*.sh")

for SCRIPT_FUNCTION in ${SCRIPT_FUNCTIONS[*]};do
    if [[ -x ${SCRIPT_FUNCTION} ]];then
        . ${SCRIPT_FUNCTION}
        export -f $(grep '^function ' ${SCRIPT_FUNCTION} | cut -d' ' -f2)
    else
        echo "${SCRIPT_NAME}: ${SCRIPT_FUNCTION} `gettext "is not executable."`"
        exit 1
    fi
done

#======================================================================
# Signals
#======================================================================

# Trap signals in order to terminate the script execution correctly
# (e.g., removing all temporal files before leaving).  Trapping the
# exit signal seems to be enough by now, since it is always present as
# part of the script execution flow. Each time the tcarlib.sh
# script is executed it will inevitably end with an EXIT signal at
# some point of its execution, even if it is interrupted in the middle
# of its execution (e.g., through `Ctrl+C').
trap tcarlib_terminateScriptExecution 0

#======================================================================
# Default Action
#======================================================================

[[ $# -eq 0 ]] && tcarlib_printUsage

#======================================================================
# Parse Command-line Arguments
#======================================================================

declare -x ARGUMENTS=''; tcarlib_getOptions "${@}"; eval set -- "${ARGUMENTS}"

#======================================================================
# Initiate Module-specific Environment
#======================================================================

declare -x SCRIPT_MODULE_NAME="${1:-help}"; shift 1
declare -x SCRIPT_MODULE_ARGUMENTS=''
for ARGUMENT in ${@};do
    SCRIPT_MODULE_ARGUMENTS="-g ${ARGUMENT} ${SCRIPT_MODULE_ARGUMENTS}"
done
