#!/bin/bash
######################################################################
#
#   tcar - The CentOS Artwork Repository
#   Copyright © 2014 The CentOS Artwork SIG
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation; either version 2 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#   Alain Reguera Delgado <alain.reguera@gmail.com>
#   39 Street No. 4426 Cienfuegos, Cuba.
#
######################################################################

# Initiate module environments inside the tcar.sh script.
function tcarlib_setModuleEnvironment {

    local ARG_MODULE_NAME=''
    local ARG_MODULE_TYPE=''
    local ARG_MODULE_ARGS=''

    # Retrieve module's name and module's type from arguments passed
    # through this function positional parameters.
    OPTIND=1
    while getopts "m:,t:,g:" OPTION "${@}"; do
        case "${OPTION}" in
            m ) ARG_MODULE_NAME="${OPTARG}" ;;
            t ) ARG_MODULE_TYPE="${OPTARG}" ;;
            g ) ARG_MODULE_ARGS="${OPTARG} ${ARG_MODULE_ARGS}" ;;
        esac
    done

    # Clean up positional parameters to reflect the fact that options
    # have been processed already.
    shift $(( ${OPTIND} - 1 ))

    # Initialize module's global counter.
    SCRIPT_MODULE_COUNT=${SCRIPT_MODULE_COUNT:-0}

    # When the last module in the chain of executed modules is the
    # same module being currently executed, don't create a new
    # position for it in the chain of modules. Instead, use the
    # information it already has from its previous execution. In order
    # for this to work, the current module must be executed as sibling
    # module of other module or itself.
    if [[ ${SCRIPT_MODULE_COUNT} -gt 0 ]];then
        if [[ ${SCRIPT_MODULE_NAMES[((${SCRIPT_MODULE_COUNT} - 1))]} == ${ARG_MODULE_NAME} ]];then
            if [[ ${ARG_MODULE_TYPE} == 'sibling' ]];then
                tcarlib_printMessage '~~~~~~~~~~~~~~~~~~~~~~~~~> : '"${SCRIPT_MODULE_NAME} ${SCRIPT_MODULE_ARGUMENT}" --as-debugger-line
                ${ARG_MODULE_NAME} ${ARG_MODULE_ARGS} ${@}
                return
            fi
        fi
    fi

    tcarlib_printMessage '=========================>: ['${SCRIPT_MODULE_COUNT}'] | '${FUNCNAME[1]} --as-debugger-line

    # Define module's base directory. This is the directory where the
    # initialization script is stored in.
    local SCRIPT_MODULE_BASEDIR=${SCRIPT_MODULES_BASEDIR}
    if [[ ${#SCRIPT_MODULE_BASEDIRS[*]} -gt 0 ]];then
        if [[ ${ARG_MODULE_TYPE} == "parent" ]];then
            SCRIPT_MODULE_BASEDIR=${SCRIPT_MODULES_BASEDIR}
        elif [[ ${ARG_MODULE_TYPE} == "sibling" ]];then
            if [[ ${SCRIPT_MODULE_TYPES[((${SCRIPT_MODULE_COUNT} - 1 ))]} == 'sibling' ]];then
                SCRIPT_MODULE_BASEDIR=${SCRIPT_MODULE_BASEDIRS[((${SCRIPT_MODULE_COUNT}-2))]}
            else
                SCRIPT_MODULE_BASEDIR=${SCRIPT_MODULE_BASEDIRS[((${SCRIPT_MODULE_COUNT}-1))]}
            fi
        else
            SCRIPT_MODULE_BASEDIR=${SCRIPT_MODULE_BASEDIRS[${SCRIPT_MODULE_COUNT}]}
        fi
    fi
    tcarlib_printMessage "SCRIPT_MODULE_BASEDIR : ${SCRIPT_MODULE_BASEDIR}" --as-debugger-line

    # Define module's name.
    SCRIPT_MODULE_NAMES[${SCRIPT_MODULE_COUNT}]=$(tcarlib_getRepoName "${ARG_MODULE_NAME:-unknown}" "-f" | cut -d '-' -f1)
    local SCRIPT_MODULE_NAME=${SCRIPT_MODULE_NAMES[${SCRIPT_MODULE_COUNT}]}
    tcarlib_printMessage "SCRIPT_MODULE_NAME : [${SCRIPT_MODULE_COUNT}]=${SCRIPT_MODULE_NAME}" --as-debugger-line

    # Define module's type.
    SCRIPT_MODULE_TYPES[${SCRIPT_MODULE_COUNT}]="${ARG_MODULE_TYPE:-parent}"
    local SCRIPT_MODULE_TYPE=${SCRIPT_MODULE_TYPES[${SCRIPT_MODULE_COUNT}]}
    tcarlib_printMessage "SCRIPT_MODULE_TYPE : ${SCRIPT_MODULE_TYPE}" --as-debugger-line

    # Define module's arguments.  This variable is used in different
    # module environments to pass positional parameters from one
    # environment to another using local definitions.
    SCRIPT_MODULE_ARGUMENTS[${SCRIPT_MODULE_COUNT}]="${ARG_MODULE_ARGS:-}"
    local SCRIPT_MODULE_ARGUMENT=${SCRIPT_MODULE_ARGUMENTS[${SCRIPT_MODULE_COUNT}]}
    tcarlib_printMessage "SCRIPT_MODULE_ARGUMENT : ${SCRIPT_MODULE_ARGUMENT}" --as-debugger-line

    # Check module's name possible values.
    tcarlib_checkModuleName

    # Define module's directory.
    SCRIPT_MODULE_DIRS[${SCRIPT_MODULE_COUNT}]=${SCRIPT_MODULE_BASEDIR}/$(tcarlib_getRepoName "${SCRIPT_MODULE_NAME}" "-d")
    local SCRIPT_MODULE_DIR=${SCRIPT_MODULE_DIRS[${SCRIPT_MODULE_COUNT}]}
    tcarlib_printMessage "SCRIPT_MODULE_DIR : ${SCRIPT_MODULE_DIR}" --as-debugger-line

    # Define module's directories not reused from module's parent
    # directory structure.
    SCRIPT_MODULE_DIRS_MODULES[${SCRIPT_MODULE_COUNT}]=${SCRIPT_MODULE_DIR}/Modules
    local SCRIPT_MODULE_DIR_MODULES=${SCRIPT_MODULE_DIRS_MODULES[${SCRIPT_MODULE_COUNT}]}
    tcarlib_printMessage "SCRIPT_MODULE_DIR_MODULES : ${SCRIPT_MODULE_DIR_MODULES}" --as-debugger-line

    SCRIPT_MODULE_DIRS_CONFIGS[${SCRIPT_MODULE_COUNT}]=${SCRIPT_MODULE_DIR}/Configs
    local SCRIPT_MODULE_DIR_CONFIGS=${SCRIPT_MODULE_DIRS_CONFIGS[${SCRIPT_MODULE_COUNT}]}
    tcarlib_printMessage "SCRIPT_MODULE_DIR_CONFIGS : ${SCRIPT_MODULE_DIR_CONFIGS}" --as-debugger-line

    # Define module's directories reused from module's parent
    # directory structure.
    SCRIPT_MODULE_DIRS_MANUALS[${SCRIPT_MODULE_COUNT}]=${SCRIPT_MODULE_DIRS[0]}/Manuals
    local SCRIPT_MODULE_DIR_MANUALS=${SCRIPT_MODULE_DIRS_MANUALS[${SCRIPT_MODULE_COUNT}]}
    tcarlib_printMessage "SCRIPT_MODULE_DIR_MANUALS : ${SCRIPT_MODULE_DIR_MANUALS}" --as-debugger-line

    SCRIPT_MODULE_DIRS_LOCALES[${SCRIPT_MODULE_COUNT}]=${SCRIPT_MODULE_DIRS[0]}/Locales
    local SCRIPT_MODULE_DIR_LOCALES=${SCRIPT_MODULE_DIRS_LOCALES[${SCRIPT_MODULE_COUNT}]}
    tcarlib_printMessage "SCRIPT_MODULE_DIR_LOCALES : ${SCRIPT_MODULE_DIR_LOCALES}" --as-debugger-line

    # Define module's initialization file.
    SCRIPT_MODULE_INIT_FILES[${SCRIPT_MODULE_COUNT}]=${SCRIPT_MODULE_DIR}/${SCRIPT_MODULE_NAME}.sh
    local SCRIPT_MODULE_INIT_FILE=${SCRIPT_MODULE_INIT_FILES[${SCRIPT_MODULE_COUNT}]}
    tcarlib_printMessage "SCRIPT_MODULE_INIT_FILE : ${SCRIPT_MODULE_INIT_FILE}" --as-debugger-line

    tcarlib_printMessage "TEXTDOMAIN: ${TEXTDOMAIN}" --as-debugger-line
    tcarlib_printMessage "TEXTDOMAINDIR: ${TEXTDOMAINDIR}" --as-debugger-line

    # Increment module's counter just before creating next module's
    # base directory.
    SCRIPT_MODULE_COUNT=$(( ${SCRIPT_MODULE_COUNT} + 1 ))

    # Define next module's base directory.
    SCRIPT_MODULE_BASEDIRS[${SCRIPT_MODULE_COUNT}]=${SCRIPT_MODULE_DIR_MODULES}

    # Check function script execution rights.
    tcarlib_checkFiles -ex ${SCRIPT_MODULE_INIT_FILE}

    # Load module-specific (function) scripts into current execution
    # environment.  Keep the tcarlib_setModuleEnvironmentScripts function
    # call after all variables and arguments definitions.
    tcarlib_setModuleEnvironmentScripts

    # Print parent module's usage information when no argument is
    # provided in the command-line.
    [[ ${#SCRIPT_MODULE_ARGUMENT} -eq 0 ]] \
        && [[ ${SCRIPT_MODULE_TYPE} == 'parent' ]] \
        && tcarlib_printUsage

    # Execute module's initialization script with its arguments.
    tcarlib_printMessage '-------------------------> : '"${SCRIPT_MODULE_NAME} ${SCRIPT_MODULE_ARGUMENT}" --as-debugger-line
    ${SCRIPT_MODULE_NAME} ${SCRIPT_MODULE_ARGUMENT}

    # Unset module-specific environment.
    tcarlib_printMessage '<------------------------- : '"${SCRIPT_MODULE_NAME} ${SCRIPT_MODULE_ARGUMENT}" --as-debugger-line
    tcarlib_unsetModuleEnvironment

    # Decrement module counter just after unset unused module
    # environments.
    SCRIPT_MODULE_COUNT=$(( ${SCRIPT_MODULE_COUNT} - 1 ))

    # Unset array and non-array variables used in this function.
    if [[ ${SCRIPT_MODULE_COUNT} -eq 0 ]];then
        unset SCRIPT_MODULE_NAMES
        unset SCRIPT_MODULE_BASEDIRS
        unset SCRIPT_MODULE_DIRS
        unset SCRIPT_MODULE_DIRS_MODULES
        unset SCRIPT_MODULE_DIRS_MANUALS
        unset SCRIPT_MODULE_DIRS_LOCALES
        unset SCRIPT_MODULE_DIRS_CONFIGS
        unset SCRIPT_MODULE_NAME
        unset SCRIPT_MODULE_DIR
        unset SCRIPT_MODULE_DIR_MODULES
        unset SCRIPT_MODULE_DIR_MANUALS
        unset SCRIPT_MODULE_DIR_LOCALES
        unset SCRIPT_MODULE_DIR_CONFIGS
    fi

    tcarlib_printMessage '<=========================: ['${SCRIPT_MODULE_COUNT}'] | '${FUNCNAME[1]} --as-debugger-line

}
